@@ -10,8 +10,7 @@
 #' @export
 #'
 #' @examples
-#' data_sd <- fast_movsd(mp_toy_data$data[,1], mp_toy_data$sub_len)
-
+#' data_sd <- fast_movsd(mp_toy_data$data[, 1], mp_toy_data$sub_len)
 fast_movsd <- function(data, window_size) {
 
   # length of the time series
@@ -56,8 +55,7 @@ fast_movsd <- function(data, window_size) {
 #' @export
 #'
 #' @examples
-#' data_avg <- fast_movavg(mp_toy_data$data[,1], mp_toy_data$sub_len)
-
+#' data_avg <- fast_movavg(mp_toy_data$data[, 1], mp_toy_data$sub_len)
 fast_movavg <- function(data, window_size) {
   data_mean <- stats::filter(data, rep(1 / window_size, window_size), sides = 2)
   return(data_mean[!is.na(data_mean)])
@@ -88,7 +86,7 @@ fast_avg_sd <- function(data, window_size) {
   data_sd2 <- pmax(data_sd2, 0)
   data_sd <- sqrt(data_sd2)
 
-  return(list(avg = data_mean, sd = data_sd))
+  return(list(avg = data_mean, sd = data_sd, sum = data_sum, sqrsum = data2_sum))
 }
 
 #' Population SD, as R always calculate with n-1 (sample), here we fix it
@@ -178,6 +176,31 @@ diff2 <- function(x, y) {
   sqrt(pmax(xx + yy - 2 * xy, 0))
 }
 
+#' Distance between two matrices
+#'
+#' Computes the Euclidean distance between rows of two matrices.
+#'
+#' @param x a `matrix`.
+#' @param y a `matrix`.
+#'
+#' @return Returns a `matrix` of size m x n if x is of size m x k and y is of size n x k.
+#' @keywords internal
+#' @noRd
+
+bubble_up <- function(data, len) {
+  pos <- len
+
+  while (pos > 0 && data[pos / 2] < data[pos]) {
+    # &&
+    #    heap->heap_[pos / 2].distance >= 0 &&
+    #   heap->heap_[pos].distance >= 0) {
+    t <- data[pos]
+    data[pos] <- data[pos / 2]
+    data[pos / 2] <- t
+    pos <- pos / 2
+  }
+}
+
 # SDTS Aux functions -----------------------------------------------------------------------------
 
 
@@ -770,20 +793,18 @@ update_class <- function(classes, new_class) {
 #' @describeIn as.matrixprofile Cast an object changed by another function back to `MatrixProfile`.
 #' @export
 #' @examples
-#'
+#' 
 #' w <- 50
 #' data <- mp_gait_data
-#' mp <- tsmp(data, window_size = w, exclusion_zone = 1/4, verbose = 0)
+#' mp <- tsmp(data, window_size = w, exclusion_zone = 1 / 4, verbose = 0)
 #' mp <- find_motif(mp)
 #' class(mp) # first class will be "Motif"
-#'
+#' 
 #' plot(mp) # plots a motif plot
-#'
+#' 
 #' plot(as.matrixprofile(mp)) # plots a matrix profile plot
-#'
-
 as.matrixprofile <- function(.mp) {
-  if (!any(class(.mp) %in% "MatrixProfile")) {
+  if (!("MatrixProfile" %in% class(.mp))) {
     stop("Error: This object cannot be a `MatrixProfile`.")
   }
 
@@ -797,7 +818,7 @@ as.matrixprofile <- function(.mp) {
 #'
 
 as.multimatrixprofile <- function(.mp) {
-  if (!any(class(.mp) %in% "MultiMatrixProfile")) {
+  if (!("MultiMatrixProfile" %in% class(.mp))) {
     stop("Error: This object cannot be a `MultiMatrixProfile`.")
   }
 
@@ -806,12 +827,26 @@ as.multimatrixprofile <- function(.mp) {
   return(.mp)
 }
 
+#' @describeIn as.matrixprofile Cast an object changed by another function back to `MultiMatrixProfile`.
+#' @export
+#'
+
+as.valmod <- function(.mp) {
+  if (!("Valmod" %in% class(.mp))) {
+    stop("Error: This object cannot be a `Valmod`.")
+  }
+
+  class(.mp) <- update_class(class(.mp), "Valmod")
+
+  return(.mp)
+}
+
 
 #' @describeIn as.matrixprofile Cast an object changed by another function back to `Fluss`.
 #' @export
 
 as.fluss <- function(.mp) {
-  if (!any(class(.mp) %in% "Fluss")) {
+  if (!("Fluss" %in% class(.mp))) {
     stop("Error: This object cannot be a `Fluss`.")
   }
 
@@ -825,7 +860,7 @@ as.fluss <- function(.mp) {
 #' @export
 
 as.chain <- function(.mp) {
-  if (!any(class(.mp) %in% "Chain")) {
+  if (!("Chain" %in% class(.mp))) {
     stop("Error: This object cannot be a `Chain`.")
   }
 
@@ -834,12 +869,24 @@ as.chain <- function(.mp) {
   return(.mp)
 }
 
+#' @describeIn as.matrixprofile Cast an object changed by another function back to `Discord`.
+#' @export
+
+as.discord <- function(.mp) {
+  if (!("Discord" %in% class(.mp))) {
+    stop("Error: This object cannot be a `Discord`.")
+  }
+
+  class(.mp) <- update_class(class(.mp), "Discord")
+
+  return(.mp)
+}
 
 #' @describeIn as.matrixprofile Cast an object changed by another function back to `Motif`.
 #' @export
 
 as.motif <- function(.mp) {
-  if (!any(class(.mp) %in% "Motif")) {
+  if (!("Motif" %in% class(.mp))) {
     stop("Error: This object cannot be a `Motif`.")
   }
 
@@ -853,7 +900,7 @@ as.motif <- function(.mp) {
 #' @export
 
 as.multimotif <- function(.mp) {
-  if (!any(class(.mp) %in% "MultiMotif")) {
+  if (!("MultiMotif" %in% class(.mp))) {
     stop("Error: This object cannot be a `MultiMotif`.")
   }
 
@@ -867,7 +914,7 @@ as.multimotif <- function(.mp) {
 #' @export
 
 as.arccount <- function(.mp) {
-  if (!any(class(.mp) %in% "ArcCount")) {
+  if (!("ArcCount" %in% class(.mp))) {
     stop("Error: This object cannot be a `ArcCount`.")
   }
 
@@ -880,7 +927,7 @@ as.arccount <- function(.mp) {
 #' @export
 
 as.salient <- function(.mp) {
-  if (!any(class(.mp) %in% "Salient")) {
+  if (!("Salient" %in% class(.mp))) {
     stop("Error: This object cannot be a `Salient`.")
   }
 
